package kimd.VNC.LookOverYourShoulder;

import java.awt.*;
import java.awt.event.*;
import java.awt.event.WindowAdapter;
import java.io.*;
import java.util.*;
import java.lang.*;
import java.net.*;
import java.lang.Object.*;

import netscape.security.PrivilegeManager;
import webwisdom.tango.*;
import netscape.javascript.JSObject;

class VNCviewerThread extends java.applet.Applet implements java.lang.Runnable {

  String userName;
  int port;
  String host;
  String pw;
  vncviewer parent;
  optionsFrame options;

  Frame f;
  ScrollPane sp;
  rfbProto rfb;

  Thread rfbThread;

  int framebufferWidth = 0;
  int framebufferHeight = 0;
  String desktopName = null;
  boolean debugApplet = false;

  public VNCviewerThread(vncviewer _parent, optionsFrame _options, String _name, int _port, String _host, String _pw) {
    parent = _parent;
    userName = _name;
    port = _port;
    host = _host;
    pw = _pw;
    options = _options;

    printLine("VNCviewerThread started");

    rfbThread = new Thread(this);
    rfbThread.start();
  }

  public void printLine(String printData) {
    if ( debugApplet ) {
      System.out.println("<VNC> " + printData);
    }
  }

  public void destroy() {
    if ( rfbThread != null ) {
      rfbThread = null;
    }
    if ( rfb != null ) {
      rfb.close();
      rfb = null;
    }
  }

  public void run() {
    try {
      connectAndAuthenticate();

      doProtocolInitialisation();

      //ImageFrame(this, rfb.desktopName, framebufferWidth, framebufferHeight, vc);
      f = new Frame("VNC - " + userName + "'s Screen");

      f.addWindowListener(new WindowAdapter() {
	public void windowClosing( WindowEvent e ) {
	  f.dispose();
	  //closeApplet();
	  destroy();
	}
	public void windowOpened(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowActivated(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
      });
      sp = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);

      setLayout(new BorderLayout());

      vncCanvas vc = new vncCanvas(parent);

      sp.add("Center", vc);
      f.add("Center", sp);

      //f.setTitle(rfb.desktopName);
      f.setTitle(userName + "(" + host + ")");
      f.pack();
      f.setSize(640, 480);
      f.setVisible(true);
      validate();

      vc.processNormalProtocol();

    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  //
  // Connect to the RFB server and authenticate the user.
  //

  void connectAndAuthenticate() throws IOException {

    boolean authenticationDone = false;

    try {
      PrivilegeManager.enablePrivilege("UniversalConnect");
    }
    catch (netscape.security.ForbiddenTargetException e) {
      System.out.println("\tFailed! Permission to connect.");
    }
    catch (Exception e) {
      System.out.println("\tFailed! Unknown exception while enabling privilege.");
    }

    while (!authenticationDone) {
      try {
	rfb = new rfbProto(host, port, parent);
	parent.rfb = rfb;
      }
      catch(IOException e) {
	printLine("check the vnc server");
	destroy();
	throw new IOException("VNC connection error - " +
			      "check hostname, port number, " + 
			      "and the vnc server");
      }

      rfb.readVersionMsg();

      printLine("RFB server supports protocol version " +
			 rfb.serverMajor + "." + rfb.serverMinor);

      rfb.writeVersionMsg();

      switch (rfb.readAuthScheme()) {

      case rfb.NoAuth:
	printLine("No authentication needed");
	authenticationDone = true;
	break;

      case rfb.VncAuth:
	byte[] challenge = new byte[16];
	rfb.is.readFully(challenge);

	if (pw.length() > 8) pw = pw.substring(0,8); // truncate to 8 chars

	if (pw.length() == 0) {
	  break;
	}

	printLine("password : " + pw);

	byte[] key = new byte[8];
	pw.getBytes(0, pw.length(), key, 0);

	for (int i = pw.length(); i < 8; i++) {
	  key[i] = (byte)0;
	}

	DesCipher des = new DesCipher(key);

	des.encrypt(challenge,0,challenge,0);
	des.encrypt(challenge,8,challenge,8);

	rfb.os.write(challenge);

	int authResult = rfb.is.readInt();

	switch (authResult) {
	case rfb.VncAuthOK:
	  printLine("VNC authentication succeeded");
	  authenticationDone = true;
	  break;
	case rfb.VncAuthFailed:
	  printLine("VNC authentication failed");
	  destroy();
	  throw new IOException("VNC authentication failed - " +
				"Invalid password");

	case rfb.VncAuthTooMany:
	  throw new IOException("VNC authentication failed - " +
				"too many tries");
	default:
	  throw new IOException("Unknown VNC authentication result " +
				authResult);
	}
	break;
      }
    }
  }

  void doProtocolInitialisation() throws IOException {
    rfb.writeClientInit();

    rfb.readServerInit();

    desktopName = rfb.desktopName;
    framebufferWidth  = rfb.framebufferWidth;
    framebufferHeight = rfb.framebufferHeight;

    printLine("Desktop name is " + desktopName);
    printLine("Desktop size is " + framebufferWidth + " * " +
		       framebufferHeight);

    setEncodings();
  }


  //
  // setEncodings() - send the current encodings from the options frame
  // to the RFB server.
  //

  void setEncodings() {
    try {
      if ((rfb != null) && rfb.inNormalProtocol) {
	rfb.writeSetEncodings(options.encodings, options.nEncodings);
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
